( cd usr/lib64 ; rm -rf libmbedcrypto.so )
( cd usr/lib64 ; ln -sf libmbedcrypto.so.6 libmbedcrypto.so )
( cd usr/lib64 ; rm -rf libmbedcrypto.so.6 )
( cd usr/lib64 ; ln -sf libmbedcrypto.so.2.25.0 libmbedcrypto.so.6 )
( cd usr/lib64 ; rm -rf libmbedtls.so )
( cd usr/lib64 ; ln -sf libmbedtls.so.13 libmbedtls.so )
( cd usr/lib64 ; rm -rf libmbedtls.so.13 )
( cd usr/lib64 ; ln -sf libmbedtls.so.2.25.0 libmbedtls.so.13 )
( cd usr/lib64 ; rm -rf libmbedx509.so )
( cd usr/lib64 ; ln -sf libmbedx509.so.1 libmbedx509.so )
( cd usr/lib64 ; rm -rf libmbedx509.so.1 )
( cd usr/lib64 ; ln -sf libmbedx509.so.2.25.0 libmbedx509.so.1 )
